#include "stdialogsettings.h"

using namespace st::widgets;

STDialogSettings::STDialogSettings(QWidget *parent)
 : QDialog(parent)
{
	m_ui = new UISettings();
	m_ui->setupUi(this);
	
	m_dialogDir = new QFileDialog();
	m_dialogDir->setFileMode(QFileDialog::DirectoryOnly);
	m_dialogDir->setModal(true);
	m_dialogDir->setDirectory( QDir( STSettings::value("limits", "destdir", QDir::homePath()).toString()  ));
	
	m_dialogDir2 = new QFileDialog();
	m_dialogDir2->setFileMode(QFileDialog::DirectoryOnly);
	m_dialogDir2->setModal(true);
	m_dialogDir2->setDirectory( QDir( STSettings::value("limits", "movedir", QDir::homePath()).toString()  ));
	
	
	connect(m_ui->toolButton_dl_save_dir, SIGNAL(clicked()),
			m_dialogDir, SLOT(show()));
	connect(m_ui->toolButton_finished_move_dir, SIGNAL(clicked()),
			m_dialogDir2, SLOT(show()));
			
	connect(m_dialogDir, SIGNAL(filesSelected ( const QStringList &)),
			this, SLOT(slot_set_dl_save_dir(const QStringList & )));
	connect(m_dialogDir2, SIGNAL(filesSelected ( const QStringList & )),
			this, SLOT(slot_set_finished_move_dir(const QStringList &)));

	connect(this, SIGNAL(accepted()),
			this, SLOT(slot_accepted()));
			
	
}


STDialogSettings::~STDialogSettings()
{
}




/*!
    \fn STDialogSettings::slot_show()
 */
void STDialogSettings::slot_show()
{
	
	QString lineEdit_finished_move_dir;
	QString lineEdit_dl_save_dir;

	bool checkBox_dl_save_dir;
	bool checkBox_finished_move_dir;
	      
	bool checkBox_add_in_paused_mode;
	bool checkBox_pre_allocate_files;
	    
	bool checkBox_dht_start;
	
	unsigned int spinBox_max_ul_speed;
	unsigned int spinBox_max_dl_speed;    

	unsigned int spinBox_max_half_connections;
	unsigned int spinBox_max_upload_connections_per_session;
	unsigned int spinBox_max_connections_per_session;
	   
	unsigned int spinBox_port_range_start;    
	unsigned int spinBox_port_range_end;
	    

	
	spinBox_max_upload_connections_per_session = 		STSettings::value("limits", "maxul", 25).toInt();
	spinBox_max_dl_speed = 		STSettings::value("limits", "ratedl", 0).toInt();
	spinBox_max_ul_speed = 		STSettings::value("limits", "rateul", 0).toInt();
	
	spinBox_max_half_connections = STSettings::value("limits", "maxhalfconnections", 0).toInt();
	spinBox_max_connections_per_session = 			STSettings::value("limits", "globalconnectionslimit", 100).toInt();
	spinBox_port_range_start = 		STSettings::value("limits", "tcpport", 6881).toInt();
	spinBox_port_range_end = 	STSettings::value("limits", "tcpport_end", 6899).toInt();

	lineEdit_dl_save_dir = 			STSettings::value("limits", "destdir", QDir::fromNativeSeparators(QDir::homePath()) + "/Desktop/torrents/").toString();
	lineEdit_finished_move_dir = 			STSettings::value("limits", "movedir", "").toString();
	checkBox_dht_start =		STSettings::value("limits", "start_dht", true).toBool();

	checkBox_dl_save_dir=		STSettings::value("limits", "checkBox_dl_save_dir", true).toBool();
	checkBox_finished_move_dir=		STSettings::value("limits", "checkBox_finished_move_dir", true).toBool();
	checkBox_add_in_paused_mode=		STSettings::value("limits", "checkBox_add_in_paused_mode", false).toBool();
	checkBox_pre_allocate_files=		STSettings::value("limits", "checkBox_pre_allocate_files", false).toBool();

	m_ui->spinBox_max_upload_connections_per_session->setValue(spinBox_max_upload_connections_per_session);
	m_ui->spinBox_max_dl_speed->setValue(spinBox_max_dl_speed);
	m_ui->spinBox_max_ul_speed->setValue(spinBox_max_ul_speed);
	
	m_ui->spinBox_max_half_connections->setValue(spinBox_max_half_connections);
	
	m_ui->spinBox_max_connections_per_session->setValue(spinBox_max_connections_per_session);
	
	m_ui->spinBox_port_range_start->setValue(spinBox_port_range_start);
	m_ui->spinBox_port_range_end->setValue(spinBox_port_range_end);
	
	m_ui->lineEdit_dl_save_dir->setText( lineEdit_dl_save_dir);
	m_ui->lineEdit_finished_move_dir->setText( lineEdit_finished_move_dir);
	
	m_ui->checkBox_dht_start->setChecked(checkBox_dht_start);
	
	
	m_ui->checkBox_dl_save_dir->setChecked(checkBox_dl_save_dir);
	m_ui->checkBox_finished_move_dir->setChecked(checkBox_finished_move_dir);
	m_ui->checkBox_add_in_paused_mode->setChecked(checkBox_add_in_paused_mode);
	m_ui->checkBox_pre_allocate_files->setChecked(checkBox_pre_allocate_files);
	
    show();

}


/*!
    \fn STDialogSettings::slot_accepted()
 */
void STDialogSettings::slot_accepted()
{

	STSettings::set_value("limits", "maxul", m_ui->spinBox_max_upload_connections_per_session->value());
	STSettings::set_value("limits", "ratedl", m_ui->spinBox_max_dl_speed->value());
	STSettings::set_value("limits", "rateul", m_ui->spinBox_max_ul_speed->value());
	
	STSettings::set_value("limits", "maxhalfconnections", m_ui->spinBox_max_half_connections->value());

	STSettings::set_value("limits", "globalconnectionslimit", m_ui->spinBox_max_connections_per_session->value());
	STSettings::set_value("limits", "tcpport", m_ui->spinBox_port_range_start->value());
	STSettings::set_value("limits", "tcpport_end", m_ui->spinBox_port_range_end->value());
	
	STSettings::set_value("limits", "destdir", m_ui->lineEdit_dl_save_dir->text());
	STSettings::set_value("limits", "movedir", m_ui->lineEdit_finished_move_dir->text());
	STSettings::set_value("limits", "movedir_bool", m_ui->checkBox_finished_move_dir->isChecked());
	
	STSettings::set_value("network", "proxy_host", m_ui->lineEdit_proxy_host->text());
	STSettings::set_value("network", "proxy_port", m_ui->spinBox_proxy_port->value());
	STSettings::set_value("network", "proxy_username", m_ui->lineEdit_proxy_username->text());
	STSettings::set_value("network", "proxy_password", m_ui->lineEdit_proxy_password->text());
	STSettings::set_value("network", "proxy_auth_bool", m_ui->checkBox_proxy_auth_bool->isChecked());
	
	STSettings::set_value("limits", "start_dht", m_ui->checkBox_dht_start->isChecked());
	
	STSettings::set_value("limits", "checkBox_dl_save_dir", m_ui->checkBox_dl_save_dir->isChecked());
	STSettings::set_value("limits", "checkBox_finished_move_dir", m_ui->checkBox_finished_move_dir->isChecked());
	STSettings::set_value("limits", "checkBox_add_in_paused_mode", m_ui->checkBox_add_in_paused_mode->isChecked());
	STSettings::set_value("limits", "checkBox_pre_allocate_files", m_ui->checkBox_pre_allocate_files->isChecked());
	
    emit signal_changed();
  
}
