#include "stsession.h"

using namespace st::core;
using namespace libtorrent;

static char const* state_str[] =
			{"queued for checking", "checking", "connecting", "downloading metadata"
			, "downloading", "finished", "seeding", "allocating"};


STSession::STSession(QObject *parent): QThread(parent)
{	
	m_mw_hidden = false;
	m_selectedTorrent = -1;
	
	connect(this, SIGNAL(signal_torrent_became_seed(libtorrent::torrent_handle )),
			this, SLOT( slot_torrent_became_seed(libtorrent::torrent_handle )), Qt::DirectConnection);
	
	#ifdef Q_WS_WIN
	boost::filesystem::path::default_name_check(boost::filesystem::no_check);
	#endif
	
	QDir *d = new QDir(STSettings::get_data_save_dir());

	if( !d->isReadable()) {
		d->mkpath(STSettings::get_data_save_dir());
	}
	
	m_session = new libtorrent::session(libtorrent::fingerprint("ST", VERSION_MAJOR, VERSION_MINOR, VERSION_REV, VERSION_TAG));
	m_session->set_severity_level(alert::info);
	
	 int port_start = STSettings::value("limits", "tcpport", 6881).toUInt();
	 int port_end = STSettings::value("limits", "tcpport_end", port_start + 10).toUInt();
	
	m_session->listen_on( std::make_pair( port_start, port_end ) );

	if( STSettings::value("limits", "start_dht", true).toBool()) {
		m_session->start_dht();
	}
	
	// read and apply settings at startup
	slot_apply_settings();
	
	startup_load_torrents();

	STSettings::remove("paused", "");
	
	start(QThread::LowestPriority);
}


STSession::~STSession()
{

}

/*!
    \fn STSession::startup_load_torrents()
 */
void STSession::startup_load_torrents()
{
	QString _tsd = STSettings::get_dir_dot_torrents();
	QString _ttd = STSettings::get_dir_dot_torrents_temp();
	
    QDir dd( _tsd );
	if( !dd.exists(_tsd)) dd.mkpath(_tsd);
	if( !dd.exists(_ttd)) dd.mkpath(_ttd);

	QStringList list2 = dd.entryList();

	slot_addTorrentFiles( list2, true, _tsd );
}


void STSession::run() {
		terminate_me = false;
		while(true && !terminate_me) {
			
			
			for(int i=0; i<20 && m_doit == false; i++) {
				msleep(100);
			}
			
			bool new_files_to_add = STSettings::update_ipc_timestamp();
			if( new_files_to_add) {
				add_new_files();
			}
			
			if( !is_mw_hidden()) {
				m_doit = false;
				if( utex.tryLock() == false) 
				{
					STff::debug(D_INF, "utex locked, from run()");
				} else 
				{
					utex.unlock();
					parse_torrents();
					emit signal_parsed();
				}
			}
			
		}

		//exec();
}

/*!
    \fn STSession::addTorrentFile(QString filename)
 */
void STSession::add_torrent_file( const QString & filename, bool startup )
{	
	
	QMutexLocker locker(&utex);

	bool compact_mode =  ! STSettings::value("limits", "checkBox_pre_allocate_files", false).toBool();
	bool start_paused = STSettings::value("limits", "checkBox_add_in_paused_mode", false).toBool();
	
	libtorrent::torrent_handle h ;
	
	if( !filename.contains(".torrent"))  return;
	
	try
	{
		
		std::ifstream in(filename.toStdString().c_str(), std::ios_base::binary);
		in.unsetf(std::ios_base::skipws);
		libtorrent::entry e = libtorrent::bdecode(std::istream_iterator<char>(in), std::istream_iterator<char>());
		
		boost::intrusive_ptr<libtorrent::torrent_info> t; 
		
		//libtorrent::torrent_info ti(e);
		
		t = new libtorrent::torrent_info(e);
		add_torrent_params _add_params;
		_add_params.ti = t;
		
		
		QString hash = sha1_hash_to_qstring(t->info_hash());
		QString fname = STSettings::get_dir_dot_torrents() + "/" + hash + ".fastresume";

		
		
		bool fast_resume_add = true;
		
		if( QFile::exists(fname)) {
			
			try {
				std::ifstream in2(fname.toStdString().c_str(), std::ios_base::binary);
				in2.unsetf(std::ios_base::skipws);
				libtorrent::entry e2 = libtorrent::bdecode(std::istream_iterator<char>(in2), std::istream_iterator<char>());
				
				//boost::filesystem::path ssave_path(STSettings::get_data_save_finished_dir().toStdString() );
				std::cout << "1";
				boost::filesystem::path ssave_path(STSettings::get_data_save_dir().toStdString() );
				std::cout << "2";
				
				_add_params.save_path = ssave_path;
				std::cout << "3";

	//			h = m_session->add_torrent(ti, ssave_path, e2, compact_mode) ;
				//h = m_session->add_torrent(_add_params) ;
			} catch( std::exception& e) {
				fast_resume_add = false;
				
			}
			// TODO 1
			if(fast_resume_add == false) {
				
				boost::filesystem::path ssave_path(STSettings::get_data_save_dir().toStdString() );
				
				_add_params.save_path = ssave_path;
	

	//			h = m_session->add_torrent(ti, ssave_path, e2, compact_mode) ;
				//h = m_session->add_torrent(_add_params) ;
			}

		} else {
			std::cout << "111";
			boost::filesystem::path ssave_path(STSettings::get_data_save_dir().toStdString() );
			
			_add_params.save_path = ssave_path;
	

	//			h = m_session->add_torrent(ti, ssave_path, e2, compact_mode) ;
			//h = m_session->add_torrent(_add_params) ;
		}
		h = m_session->add_torrent(_add_params) ;

			
		QFileInfo finfo(filename);
		
		if( !startup) {
			QString dest_file = STSettings::get_dir_dot_torrents() + "/" + finfo.fileName();
			QFile::copy( filename, dest_file );
			m_hash_filename_map[h.info_hash()] = STSettings::get_dir_dot_torrents() + "/" + finfo.fileName();

		} else {
			m_hash_filename_map[h.info_hash()] = filename;
		}
		
		if(startup) {
			start_paused = STSettings::value("paused", hash, false).toBool();
		}
		if( start_paused ) h.pause();
		
		signal_alert_send( "added " +finfo.fileName() );
	} catch (std::exception& e) 
	{
		signal_debug_send( QString(e.what()));
	}
	
	
}


/*!
    \fn STSession::addTorrentFiles(const QStringList & filenames)
 */
void STSession::slot_addTorrentFiles(const QStringList & filenames, bool startup, QString basedir )
{
	int ii=0;
	for(; filenames.size() > ii; ++ii) {
		add_torrent_file(  basedir +  filenames.at(ii).toLocal8Bit().constData(), startup );
	}	
	slot_doit();
}

/*!
    \fn STSession::removeTorrent(int tn, bool deleteData)
 */
void STSession::slot_removeTorrent()
{
	QMutexLocker locker(&utex);
	if(m_selectedTorrent == "") return;
	
		torrent_handle h = get_selected_torrent_handle();
		
		if( !h.is_valid()) return;
		
		m_session->remove_torrent( h );
		
		slot_doit();
		
		QFile::remove(m_hash_filename_map[h.info_hash()]);
		
}

/*!
    \fn STSession::pauseTorrent(sha_hash const& ih)
 */
void STSession::slot_pauseTorrent()
{
	QMutexLocker locker(&utex);
	
	if(m_selectedTorrent == "") return;
	
	torrent_handle h = get_selected_torrent_handle();
	
	if( !h.is_valid()) 	return;
	
	h.pause();
	STSettings::set_value("paused", sha1_hash_to_qstring(h.info_hash()), true );
	slot_doit();
}

/*!
    \fn STSession::resumeTorrent(int tn)
 */
void STSession::slot_resumeTorrent()
{
	QMutexLocker locker(&utex);
	
	if(m_selectedTorrent == "") return;

	torrent_handle h = get_selected_torrent_handle();
	
	if( !h.is_valid()) 	return;
	
	h.resume();
	STSettings::set_value("paused", sha1_hash_to_qstring(h.info_hash()), false );
	slot_doit();
}

/*!
    \fn STSession::slot_apply_settings()
 */
void STSession::slot_apply_settings()
{
	QMutexLocker locker(&utex);
	
	int maxdl, maxul, ratedl, rateul;
	int maxhalfconnections;
	double ratio = 0;
	int gcl, tcpport, tcpport_end;
	QString ddir, mdir;
	bool start_dht;
	
	
	maxdl = STSettings::value("limits", "maxdl", -1).toInt() ;
	maxul = STSettings::value("limits", "maxul", 25).toInt();
	ratedl = STSettings::value("limits", "ratedl", -1).toInt()  ;
	rateul = STSettings::value("limits", "rateul", -1).toInt() ;
	maxhalfconnections = STSettings::value("limits", "maxhalfconnections", -1).toInt();
	gcl = STSettings::value("limits", "globalconnectionslimit", 800).toInt();
	tcpport = STSettings::value("limits", "tcpport", 6881).toInt();
	tcpport_end = STSettings::value("limits", "tcpport_end", 6899).toInt();
	ratio = STSettings::value("limits", "ratio", 0).toDouble();
	start_dht = STSettings::value("limits", "start_dht", true).toBool();
	
	ddir = STSettings::value("limits", "destdir", "").toString();
	mdir = STSettings::value("limits", "movedir", "").toString();

	// max upload speed
	int trate = -1;
	if(rateul != -1) trate = rateul * 1024;
	if(rateul <= 0) trate = -1;
 	m_session->set_upload_rate_limit(trate);

	// max download speed
	trate = -1;
	if(ratedl != -1) trate = ratedl * 1024;
	if(ratedl <= 0) trate = -1;
	m_session->set_download_rate_limit(trate);

	// max half conn
	if(maxhalfconnections <= 0) maxhalfconnections = -1;
	m_session->set_max_half_open_connections(maxhalfconnections );

	// max global conn
	m_session->set_max_connections(gcl);

	m_session->set_max_uploads(maxul);
	
	if(start_dht) {

		//m_session->start_dht();
	} else {
		m_session->stop_dht();
	}
}


/*!
    \fn STSession::getTorrentHandle(int n)
 */
const libtorrent::torrent_handle  STSession::get_selected_torrent_handle() const
{
	std::vector<torrent_handle> lh = m_session->get_torrents();
	
	std::vector<torrent_handle>::const_iterator i = lh.begin();
	
	for( int _count=0 ; i != lh.end() ; i++, _count++) {	
		std::stringstream _ss1;
		_ss1 << i->info_hash();
		QString hash(_ss1.str().c_str() );
		// = QString(_ss1.str().c_str() );
	
		if( hash == m_selectedTorrent) {
			
			torrent_handle h = *i;
			return h;
		}
	}
	return torrent_handle();
}



/*!
    \fn STSession::parseTorrents()
 */
void STSession::parse_torrents()
{
	QMutexLocker locker(&utex);
	
	parse_alerts();
	parse_session_stats();
	

    std::vector<torrent_handle> lh = m_session->get_torrents();

    m_map_torrents.clear();

    std::vector<torrent_handle>::const_iterator i = lh.begin();
    int _count=0;

    for( ; i != lh.end() ; i++, _count++) {
    	torrent_handle h = *i;
    	libtorrent::torrent_status s = h.status();
        libtorrent::torrent_info ti = h.get_torrent_info();;

		QString hash = sha1_hash_to_qstring(h.info_hash());

		if( hash == m_selectedTorrent) {
			
			if( m_mi_tab == 0)
				parse_torrent_status(h);
				
			if( m_mi_tab == 1)
				parse_torrent_files(h);
						
			if( m_mi_tab == 2)
				parse_torrent_peers(h);
		}
		
    	// torrent name 0 
    	m_map_torrents[hash].append(QString(h.get_torrent_info().name().c_str()));
    	// progress 1
    	m_map_torrents[hash].append(QString("%1 %").arg(s.progress*100, 0, 'f', 2));

		if(h.is_paused()) {
			m_map_torrents[hash].append(QString("paused"));
		} else {
			m_map_torrents[hash].append(QString( state_str[s.state] ));
		}
    	
    	// total size 3
    	m_map_torrents[hash].append( bytes_to_human_readable(ti.total_size()));
    	// total done 4
    	m_map_torrents[hash].append( bytes_to_human_readable(s.total_done));
    	
    	// 5 6
		m_map_torrents[hash].append(QString::number(s.num_peers));
		m_map_torrents[hash].append(QString::number(s.num_seeds));
		
		// 7
		m_map_torrents[hash].append(QString("%1").arg(s.distributed_copies, 0, 'f', 2));
		
		// 8 9
		m_map_torrents[hash].append(QString("%1 kB/s").arg( s.download_rate/1024, 0, 'f', 2));
		m_map_torrents[hash].append(QString("%1 kB/s").arg( s.upload_rate/1024, 0, 'f', 2));
		
		/*
		float ratio = 0;
		if( s.total_payload_upload != 0 )
						ratio = s.total_payload_download / s.total_payload_upload;
		*/

		boost::posix_time::time_duration t = s.next_announce;
		std::string t_ss = boost::posix_time::to_simple_string(t);
		std::string current_tracker = s.current_tracker;
		
		
		// 10 11 12
		m_map_torrents[hash].append(hash);
		m_map_torrents[hash].append(QString(t_ss.c_str() ));
		m_map_torrents[hash].append(QString(current_tracker.c_str() ));
		
		//qDebug() << m_map_torrents[hash];
	
    }
}

/*!
    \fn STSession::parseTorrentFiles()
 */
void STSession::parse_torrent_files(torrent_handle h)
{
	if( !h.is_valid()) return;
	
	m_map_files.clear();
	
	if (h.has_metadata())
	{
		std::vector<float> file_progress;
		h.file_progress(file_progress);
		torrent_info const& info = h.get_torrent_info();
	
		int _icount=0;
		int k = 0;
		for (int i = 0; i < info.num_files(); ++i, _icount++)
		{	
			
			//for( int j=0; j< 10; j++) {
			k++;
				m_map_files[k].append( QString("%1").arg( _icount ));
				m_map_files[k].append( QString(info.file_at(i).path.string().c_str()));
				m_map_files[k].append( QString("%1 %").arg( (file_progress[i] + 0.00)*100, 0, 'f', 2 ));
			//}
				
		}
	}
}

/*!
    \fn STSession::parseTorrentPeers()o
 */
void STSession::parse_torrent_peers(torrent_handle h)
{
	if( !h.is_valid()) return;
	
	m_map_peers.clear();
	
	std::vector<peer_info> peers;
	
	h.get_peer_info(peers);
	
	int _icount = 0;
	
	for (std::vector<peer_info>::const_iterator i = peers.begin();
						i != peers.end(); ++i, _icount++)
	{
		asio::ip::address _address =  i->ip.address();

		m_map_peers[_icount].append( QString("%1").arg(_icount)  );
		m_map_peers[_icount].append( QString(_address.to_string().c_str()) );
		m_map_peers[_icount].append( QString(i->client.c_str()) );
		m_map_peers[_icount].append( QString("%1 kB/s").arg(i->down_speed/1024, 0, 'f', 2) );
		m_map_peers[_icount].append( QString("%1 kB/s").arg(i->up_speed/1024, 0, 'f', 2) );
	}
}

/*!
    \fn STSession::parseTorrentStatus(torrent_handle h)
 */
void STSession::parse_torrent_status(torrent_handle h)
{
	if( !h.is_valid()) return;
	
	m_map_status.clear();
	map_string_stringlist _map;
	
	libtorrent::torrent_status status = h.status();
	libtorrent::torrent_info info = h.get_torrent_info();;
	
	// lets get values from torrent_status
	
	boost::posix_time::time_duration t;
	std::string t_ss;
	
	t = status.next_announce;
	t_ss = boost::posix_time::to_simple_string(t);
	_map["announce next"].append( QString(t_ss.c_str()));
		
	t = status.announce_interval;
	t_ss = boost::posix_time::to_simple_string(t);
	_map["announce interval"].append( QString(t_ss.c_str()));
	
	_map["tracker current"].append( QString(status.current_tracker.c_str()));

	_map["total download"].append( QString::number(status.total_download ));
	_map["total upload"].append( QString::number(status.total_upload ));
	
	_map["total payload download"].append( QString::number(status.total_payload_download ));
	_map["total payload upload"].append( QString::number(status.total_payload_upload ));
	
	_map["total failed bytes"].append( QString::number(status.total_failed_bytes ));
	_map["total redundant bytes"].append( QString::number(status.total_redundant_bytes ));
	
	_map["download rate"].append( QString::number(status.download_rate ));
	_map["upload rate"].append( QString::number(status.upload_rate ));
	
	_map["download payload rate"].append( QString::number(status.download_payload_rate ));
	_map["upload payload rate"].append( QString::number(status.upload_payload_rate ));
	
	_map["num peers"].append( QString::number(status.num_peers ));
	_map["num complete"].append( QString::number(status.num_complete ));
	_map["num incomplete"].append( QString::number(status.num_incomplete ));
	_map["num pieces"].append( QString::number(status.num_pieces ));
	
	_map["total done"].append( QString::number(status.total_done ));
	_map["total wanted done"].append( QString::number(status.total_wanted_done ));
	_map["total wanted"].append( QString::number(status.total_wanted ));
	
	_map["num seeds"].append( QString::number(status.num_seeds ));
	
	_map["distributed copies"].append( QString::number(status.distributed_copies ));
	_map["block size"].append( QString::number(status.block_size ));
	//_map[""].append( QString::number(status. ));
        /*
        const std::vector<bool>* pieces;
        */

	// now lets parse torrent_info
	
	_map["num files"].append( QString::number( info.num_files()));
	
	std::vector<std::string> v = info.url_seeds();
	std::vector<std::string>::const_iterator i = v.begin();
	
	_map["url seeds"].clear();
	for( ; i != v.end(); i++) {
		_map["url seeds"].append( 	QString( (*i).c_str() ) );
	}
	
	_map["total size"].append( QString::number(info.total_size()) );
	_map["piece length"].append( QString::number(info.piece_length()) );

	
	_map["torrent name"].append( QString(info.name().c_str()));
	_map["torrent comment"].append( QString(info.comment().c_str()));
	_map["torrent is private"].append( info.priv() ? "yes" : "no" );
	
	std::vector<std::pair<std::string, int> > v2 = info.nodes();
	std::vector<std::pair<std::string, int> >::const_iterator i2 = v2.begin();
	_map["dht nodes"].clear();
	for( ; i2 != v2.end(); i2++) {
		_map["dht nodes"].append( 	QString( (i2->first).c_str() ) + ":" + QString::number( (i2->second)) );
	}
	
	std::vector<announce_entry> v_trackers = info.trackers();
	std::vector<announce_entry>::const_iterator i3 = v_trackers.begin();
	for( ; i3 != v_trackers.end(); i3++) {
	_map["trackers"].clear();
		_map["trackers"].append( QString(i3->url.c_str()) );
	}
	// torrent_handle
	
	// waiting 0.13 libtorrent
	//_map["upload limit"].append( QString::number( h.upload_limit()));
	//_map["download limit"].append( QString::number( h.download_limit()));
	
	m_map_status = _map;
}


/*!
    \fn STSession::parseSessionStats()
 */
void STSession::parse_session_stats()
{	
	map_string_string _map;
	m_map_session_stats.clear();
	
	_map["upload_rate_limit"] = bytes_to_human_readable(m_session->upload_rate_limit());
	_map["download_rate_limit"] = bytes_to_human_readable(m_session->download_rate_limit() );
	
	_map["num_uploads"] = QString::number(m_session->num_uploads() );
	_map["num_connections"] = QString::number(m_session->num_connections() );
	
	libtorrent::session_status status = m_session->status();
	
	_map["upload_rate"] = bytes_to_human_readable(status.upload_rate);
	_map["download_rate"] = bytes_to_human_readable(status.download_rate);
	
	_map["total_download"] = bytes_to_human_readable(status.total_download);
	_map["total_upload"] = bytes_to_human_readable(status.total_upload);
	
	
	_map["num_peers"] = QString::number(status.num_peers);
	
	_map["dht_nodes"] = QString::number(status.dht_nodes);
	//_map["dht_cache_nodes"] = QString::number(status.dht_cache_nodes);
	_map["dht_torrents"] = QString::number(status.dht_torrents);
	
	_map["dht_status"] = "on";
	if( status.dht_nodes == 0) {
		_map["dht_status"] = "off";
	}


	_map["is_listening"] = "no";
	if( m_session->is_listening()) _map["is_listening"] = "yes";
	_map["listen_port"] = QString::number(m_session->listen_port());

	//entry dht_state() const;
	
	m_map_session_stats = _map;
	
	/*
	qDebug() << "";
	qDebug() << _map.keys();
	qDebug() << _map.values();
	qDebug() << "";
	*/
}



/*!
    \fn STSession::setSelectedTorrent(sha1_hash hash, int mi_tab)
 */
void STSession::slot_setSelectedTorrentHash(QString st)
{

    m_selectedTorrent = st;
    slot_doit();
}

void STSession::slot_setSelectedMITab(int mi_tab) {
	m_mi_tab = mi_tab;
	slot_doit();
}

/*!
    \fn STSession::slot_doit()
 */
void STSession::slot_doit()
{	
	m_doit = true;
}

/*!
    \fn STSession::aboutToQuit()
 */
void STSession::about_to_quit()
{

	std::vector<torrent_handle> lh = m_session->get_torrents();
    std::vector<torrent_handle>::iterator i = lh.begin();

    bool is_paused;
    
    for( ; i != lh.end() ; i++) {
    	torrent_handle h = *i;
    	if(h.is_valid()) {
    		
    		is_paused = h.is_paused();
			h.pause();
			if (h.has_metadata())
			{
				entry data = h.write_resume_data();
				
				QString s;
				s = sha1_hash_to_qstring(h.info_hash()) + QString(".fastresume");

				boost::filesystem::ofstream out( (STSettings::get_dir_dot_torrents() + "/" + s).toStdString() , std::ios_base::binary);
				STSettings::set_value("paused", sha1_hash_to_qstring(h.info_hash()), is_paused );
				out.unsetf(std::ios_base::skipws);
				bencode(std::ostream_iterator<char>(out), data);
			}
			//m_session->remove_torrent(h);
		} else {
			STff::debug(D_CRIT, "h is not valid");
		}
    }
	
	terminate_me = true;
	//wait();

	
	STff::debug(D_CRIT, "stting ipc_timestamp to 1");
	STSettings::update_ipc_timestamp(1);
    /// @todo implement me
}


/*!
    \fn STSession::get_map_peers() const
 */
map_int_stringlist STSession::get_map_peers() 
{
    return m_map_peers;
}

map_int_stringlist STSession::get_map_files() 
{

    return m_map_files;

}
map_string_stringlist STSession::get_map_torrents() 
{
	return m_map_torrents;
}

map_string_stringlist STSession::get_map_status() 
{
	return m_map_status;
}


/*!
    \fn STSession::mutex_lock(bool b)
 */
bool STSession::mutex_tryLock()
{
	return utex.tryLock();
}

void STSession::mutex_unlock()
{
	utex.unlock();
}


/*!
    \fn STSession::set_mw_hidden();
 */
void STSession::set_mw_hidden(bool b)
{
    m_mw_hidden = b;
}


/*!
    \fn STSession::is_mw_hidden()
 */
bool STSession::is_mw_hidden()
{
    return m_mw_hidden;
}


/*!
    \fn STSession::add_new_files()
 */
void STSession::add_new_files()
{
    QDir _tdir( STSettings::get_dir_dot_torrents_temp());
    
    QStringList _tdir_filters;
    _tdir_filters << "*.torrent";
    QStringList _tdir_files = _tdir.entryList(_tdir_filters);
    
     for(int __j = 0; __j  < _tdir_files.size() ; __j++) {
		QString _fn1 = STSettings::get_dir_dot_torrents_temp() + "/" + _tdir_files.at(__j);
		add_torrent_file( _fn1, false);
				
		STff::debug(D_INF, "\t run step 8");
		if(!QFile::remove(_fn1))
		STff::debug(D_CRIT, "failed to remove _fn1=" + _fn1);
	}

}




/*!
    \fn STSession::get_map_session_stats()
 */
map_string_string STSession::get_map_session_stats()
{
	return m_map_session_stats;
}


/*!
    \fn STSession::bytes_to_human_readable(long long bytes)
 */
QString STSession::bytes_to_human_readable(long long bytes)
{
	if( bytes > 1024*1024*1024) {
		return QString("%1 GB").arg( (float)bytes / (1024*1024*1024), 0, 'f', 2) ;
	} else
	if(bytes > 1024*1024) {
		return QString("%1 MB").arg( (float)bytes / (1024*1024), 0, 'f', 2) ;
	} else
	if( bytes > 1024) {
		return QString("%1 KB").arg( (float)bytes / (1024), 0, 'f', 2) ;
	} else {
		return QString("%1 B").arg( (float)bytes, 0, 'f', 2) ;
	}
}


/*!
    \fn STSession::parse_alerts()
 */
void STSession::parse_alerts()
{
	std::auto_ptr<alert> a;
	a = m_session->pop_alert();
	std::string now = QDateTime::currentDateTime().toString().toStdString();

	 while (a.get())
	{
		if (torrent_finished_alert* p = dynamic_cast<torrent_finished_alert*>(a.get()))
		{
				//p->handle.set_max_connections(60);
	
				// write resume data for the finished torrent
				torrent_handle h = p->handle;
				emit signal_torrent_became_seed(h);
				
				events.enqueue(now + ": "
						+ p->handle.get_torrent_info().name() + ": " + a->message());
		}
		else if (peer_error_alert* p = dynamic_cast<peer_error_alert*>(a.get()))
		{
				events.enqueue(now + ": " + identify_client(p->pid)
						+ ": " + a->message());
		}
		else if (invalid_request_alert* p = dynamic_cast<invalid_request_alert*>(a.get()))
		{
				events.enqueue(now + ": " + identify_client(p->pid)
						+ ": " + a->message());
		}
		else if (tracker_warning_alert* p = dynamic_cast<tracker_warning_alert*>(a.get()))
		{
				events.enqueue(now + ": tracker message: " + p->message());
		}
		else if (tracker_reply_alert* p = dynamic_cast<tracker_reply_alert*>(a.get()))
		{
				events.enqueue(now + ": " + p->message() + " ("
						+ boost::lexical_cast<std::string>(p->num_peers) + ")");
		}
		else if (url_seed_alert* p = dynamic_cast<url_seed_alert*>(a.get()))
		{
				events.enqueue(now + ": web seed '" + p->url + "': "+ p->message());
		}
		else
		{
				events.enqueue(now + ": " + a->message());
		}
	
		//if (events.size() >= 10) events.pop_front();
		while (!events.isEmpty()) {
			QString s = QString(events.dequeue().c_str());
			signal_alert_send(s);

     	}
		a = m_session->pop_alert();
	}

}


/*!
    \fn STSession::sha1_hash_to_qstring(const & sha1_hash hash)
 */
QString STSession::sha1_hash_to_qstring(const sha1_hash &hash)
{
    std::stringstream _ss1;
	_ss1 << hash;
	QString ti_hash = QString(_ss1.str().c_str() );
	
	return ti_hash;
}


/*!
    \fn STSession::slot_torrent_became_seed(torrent_handle &h)
 */
void STSession::slot_torrent_became_seed(libtorrent::torrent_handle h)
{
	entry data = h.write_resume_data();
	QString s;
	s = sha1_hash_to_qstring(h.info_hash()) + QString(".fastresume");
	boost::filesystem::ofstream out( (STSettings::get_dir_dot_torrents() + "/" + s).toStdString() , std::ios_base::binary);
	out.unsetf(std::ios_base::skipws);
	bencode(std::ostream_iterator<char>(out), data);
	
	
}
