#ifndef STSESSION_H
#define STSESSION_H

#include <iostream>
#include <fstream>
#include <iterator>
#include <exception>

#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/filesystem/exception.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/bind.hpp>

#include "libtorrent/entry.hpp"
#include "libtorrent/bencode.hpp"
#include "libtorrent/session.hpp"
#include "libtorrent/identify_client.hpp"
#include "libtorrent/alert_types.hpp"
#include "libtorrent/ip_filter.hpp"

#include <QMap>
#include <QStringList>
#include <QThread>
#include <QObject>

#include <QDir>
#include <QFile>
#include <QFileInfo>


#include <QList>
#include <QTime>
#include <QFile>
#include <QMutex>
#include <QMutexLocker>
#include <QWaitCondition>

#include <QQueue>

#include <QTextStream>

#include "stsettings.h"
#include "stff.h"

using namespace libtorrent;
namespace st {
namespace core {
/**
	@author sfinks <sfinks2@gmail.com>
*/


class STSession : public QThread {
	Q_OBJECT
public:
    STSession(QObject *parent = 0);
    ~STSession();
    void about_to_quit();
    
    map_int_stringlist get_map_peers() ;
    map_int_stringlist get_map_files() ;
    map_string_stringlist get_map_status() ;
    
    map_string_stringlist get_map_torrents() ;
	
	bool mutex_tryLock();
	void mutex_unlock();
    void set_mw_hidden(bool b);
    bool is_mw_hidden();
    void add_new_files();
    map_string_string get_map_session_stats();
    void parse_alerts();
    
	//std::deque<std::string> events;
	QQueue<std::string> events;
	    
private: // private functions
	void add_torrent_file(const QString & filename, bool startup = false);
	const libtorrent::torrent_handle get_selected_torrent_handle() const;
	
	void run();
    void parse_torrents();
    void parse_torrent_files(torrent_handle h);
    void parse_torrent_peers(torrent_handle h);
    void startup_load_torrents();

	QString sha1_hash_to_qstring(const  sha1_hash &hash);
	
    void parse_torrent_status(torrent_handle h);
    void parse_session_stats();
    QString bytes_to_human_readable(long long bytes);
    
    
private: // private variables
	libtorrent::session *m_session;
	
	QString m_selectedTorrent;
	int m_mi_tab;

	QMap<sha1_hash, QString> m_hash_filename_map;
	
	map_string_stringlist m_map_torrents;
	map_int_stringlist m_map_files, m_map_peers;
	map_string_stringlist m_map_status;
	
	map_string_string m_map_session_stats;
	
	bool m_doit;
	QMutex utex;	
	
	bool m_mw_hidden;
	bool terminate_me;
	
public slots:
    void slot_addTorrentFiles(const QStringList & filenames, bool startup = false, QString basedir = QString(""));
    
    void slot_removeTorrent();
    void slot_pauseTorrent();
    void slot_resumeTorrent();

    void slot_setSelectedTorrentHash(QString st);
    void slot_setSelectedMITab(int mi_tab); // 0 - status, 1 - files, 2 - peers
    void slot_apply_settings();
    
    void slot_doit();
    
private slots:
	void slot_torrent_became_seed(libtorrent::torrent_handle h);
	
signals:
	void signal_parsed();
	void signal_alert_send(const QString &s);
	void signal_debug_send(const QString &s);
	void signal_torrent_became_seed(libtorrent::torrent_handle h);
    
};
}
}
#endif
