#include "stwidgetdownloads.h"

STWidgetDownloads::STWidgetDownloads(QWidget *parent)
 : STWidget(parent)
{
	m_ui = new UIDownloads();
	m_ui->setupUi(this);
		
	m_ui->treeView->setSortingEnabled(true);

	m_model = new QStandardItemModel();
		
	
	create_popup_menu();
	
	m_ui->treeView->setModel( m_model );
	
	create_treeview_header();	
	
	connect( m_ui->treeView->selectionModel(), SIGNAL(selectionChanged( const QItemSelection &, const QItemSelection &)),
                        this, SLOT(slot_selectionChanged( const QItemSelection &, const QItemSelection &)));
	connect( m_ui->treeView, SIGNAL( signal_rightMousePressed(const QPoint &)),
                        this, SLOT( slot_popup(const QPoint &)));

}

STWidgetDownloads::~STWidgetDownloads()
{

}

/*!
    \fn STWidgetDownloads::getTreeView()
 */
QTreeView* STWidgetDownloads::get_treeWidget()
{
    return m_ui->treeView;
}
QListWidget* STWidgetDownloads::get_listWidget() {
	return m_ui->listWidget;
}


void STWidgetDownloads::slot_selectionChanged( const QItemSelection & selected, const QItemSelection & deselected ) 
{

	QString hash;
	if( m_ui->treeView->selectionModel()->selectedRows().size() == 1) {

	hash =
		  ((STStandardItem *)m_model->itemFromIndex(selected.indexes().at(10)))->text();

	} else {
		hash = "";
	}
	
	emit signal_selected(  hash );
}

/*!
    \fn STWidgetDownloads::slot_popup(const QPoint &p)
 */
void STWidgetDownloads::slot_popup(const QPoint &p)
{
    m_popup->popup(p);
}

/*!
    \fn STWidgetDownloads::createPopupMenu()
 */
void STWidgetDownloads::create_popup_menu()
{
	m_popup = new QMenu("Menu", m_ui->treeView);
	
	m_actionResume = m_popup->addAction("resume/start");
	m_actionPause = m_popup->addAction("pause");
	m_popup->addSeparator();
	m_actionRemove = m_popup->addAction("remove");
	
	connect( m_ui->treeView, SIGNAL( signal_rightMousePressed(const QPoint &)),
			this, SLOT( slot_popup(const QPoint &)));
}



/*!
    \fn STWidgetDownloads::createTreeViewHeader()
 */
void STWidgetDownloads::create_treeview_header()
{
	m_hv = m_ui->treeView->header();

	
	QStringList h;
	h << "name" << "progress" << "status"
			<< "total size" << "total done" 
			<< "peers"
			<< "seeders"
			<< "dist copies"
			<< "download rate"
			<< "upload rate" 
			<< "hash"
			<< "next announce";

	
	m_model->setHorizontalHeaderLabels(h);
	
	int default_width;
	for(int i=0; i< h.size(); i++) {
	
			default_width = 75;
			if(i==0) 
				default_width = 200;
	
			int width = STSettings::value("widget_downloads", "header_width_"+QString::number(i), default_width).toInt();

			m_hv->resizeSection(i, width);
	}

	connect( m_hv, SIGNAL(sectionResized ( int , int , int  ) ),
			this, SLOT( slot_write_header_conf() ));

}

void STWidgetDownloads::slot_update_model(map_string_stringlist l)
{
	QStringList labels = get_labels();
	
	QList<QString> list_of_map_keys = l.keys();
	if( list_of_map_keys == m_list_of_map_keys_previous && labels == m_labels_previous) {
		
	} else {	
	
		for(int i=0 ; i < m_model->rowCount() ; i++) {
			QString hash = ((STStandardItem *)m_model->item(i, 10))->text();
			if( l.keys().contains( hash) == false ) {

				m_model->removeRow(i);
				i=0;
			}
		}
		
		QMapIterator<QString, QStringList> i(l);
		
		while (i.hasNext()) {
			i.next();
			QList<QStandardItem *> qlsi;
	
			QString label_str;
			if( i.value().at(2) == "downloading") {	
				label_str = "down";
			} else 
			if( i.value().at(2) == "seeding") {	
				label_str = "seed";
			} else {
				label_str = "other";
			}
			
			bool already_in_model = false;
			for(int j=0 ; j < m_model->rowCount() ; j++) {
					QString hash = ((STStandardItem *)m_model->item(j, 10))->text();
					if( i.key() == hash ) already_in_model = true;
			}
				
			if( (labels.contains(label_str) || labels.contains("all")) && !already_in_model ) {	
				for( int j=0; j<12; j++) {
						if(j == 10) {
							qlsi.append(new STStandardItem( i.key() ));
						} else if(j == 2) {
							qlsi.append(new STStandardItem( i.value().at(2) ));
						} else {
							qlsi.append(new STStandardItem(QString::number(j)));
						}
				}
				m_model->appendRow( qlsi  );
			}
		}
	} // if

	for(int i=0 ; i < m_model->rowCount() ; i++) {
		QString hash = ((STStandardItem *)m_model->item(i, 10))->text();	
		QString label_str_raw =l[hash].at(2);	
		
		QString label_str;
		if( label_str_raw == "downloading") {	
			label_str = "down";
		} else 
		if( label_str_raw == "seeding") {	
			label_str = "seed";
		} else {
			label_str = "other";
		}
		if( !(labels.contains(label_str) || labels.contains("all"))) {
			m_model->removeRow(i);
			i=0;
		} else {
			for( int j=0; j<12; j++) {
					if( l[hash].size() > j) {
	
							((STStandardItem *)m_model->item(i, j))->setText( l[hash].at(j) );
					} else {
					}
			}
		}
    }

	sort();
	m_list_of_map_keys_previous = list_of_map_keys;
	m_labels_previous = labels;
	
	/*
	if( m_ui->treeView->selectionModel()->selectedIndexes().size() ==1) {
		m_ui->treeView->scrollTo( m_ui->treeView->selectionModel()->selectedIndexes().at(0));
	}
	*/
}



/*!
    \fn STWidgetDownloads::getLabels()
 */
 
QStringList STWidgetDownloads::get_labels() const
{
    QList<QListWidgetItem *> lwitems = m_ui->listWidget->selectedItems ();
	QStringList lwitems_str;

	for(int k=0; k < lwitems.size(); k++) {
			lwitems_str.append( lwitems.at(k)->text());
	}

	return lwitems_str;
	/*
	QStringList l;
	 l << "all";
	 return l;
	 */
}


/*!
    \fn STWidgetDownloads::slot_write_header_conf()
 */
void STWidgetDownloads::slot_write_header_conf()
{

	for(int i=0; i< m_hv->count() ; i++) {
			STSettings::set_value("widget_downloads", 
					"header_width_"+QString::number(i), 
					m_ui->treeView->columnWidth(i));
	}
}
