#include "stwidgetmainwindow.h"

STWidgetMainWindow::STWidgetMainWindow(QWidget *parent)
 : QMainWindow(parent)
{
	m_ui = new UIMainWindow();
	m_ui->setupUi(this);

	
	actionHide = new QAction("minimize", this);
	actionShow = new QAction("restore", this);
	actionBHide = new QAction("diable when hiding main window", this);

	
	connect( actionHide, SIGNAL(triggered()),
			this, SLOT( hide() ));
	connect( actionShow, SIGNAL(triggered()),
			this, SLOT( show() ));
	connect( actionBHide, SIGNAL(triggered()),
			this, SLOT( slot_balloon_on_hide() ));
	
	m_trayMenu = new QMenu(this);
	m_trayMenu->addAction(actionShow);
	m_trayMenu->addAction(actionHide);
	
	m_trayMenu->addSeparator();
	
	QMenu *bmenu = new QMenu("disable/enable balloons", m_trayMenu);
	m_trayMenu->addMenu( bmenu);
	
	bmenu->addAction(actionBHide);
	m_trayMenu->addSeparator();
	
    m_trayMenu->addAction(m_ui->action_Quit);
	
	m_trayIcon = new QSystemTrayIcon( QIcon(QString::fromUtf8(":/bitcons/bitcons/type.xpm")), this);
    m_trayIcon->setContextMenu(m_trayMenu);

	m_trayIcon->show();
	
	read_settings();
	
	// TODO: rewrite this
	m_balloonOnHideEnabled = !m_balloonOnHideEnabled;
	this->slot_balloon_on_hide();
	
	#ifdef Q_WS_MAC
                m_ui->actionMac->setEnabled(true);
	#endif
	#ifdef Q_WS_WIN
		if(QSysInfo::WindowsVersion == QSysInfo::WV_XP) {
			m_ui->actionWindows_XP->setEnabled(true);
		}
		if(QSysInfo::WindowsVersion == QSysInfo::WV_VISTA) {
			m_ui->actionWindows_Vista->setEnabled(true);
		}
	#endif
 
	connect( m_ui->actionMac, SIGNAL(triggered()),
				this, SLOT(slot_StyleMac() ));
	connect( m_ui->actionMotif, SIGNAL(triggered()),
				this, SLOT(slot_StyleMotif() ));
	connect( m_ui->actionPlastique, SIGNAL(triggered()),
				this, SLOT(slot_StylePlastique() ));
				
	connect( m_ui->actionCleanlooks, SIGNAL(triggered()),
				this, SLOT(slot_StyleCleanlooks()));
	connect( m_ui->actionCDE, SIGNAL(triggered()),
				this, SLOT(slot_StyleCDE()));
	connect( m_ui->actionWindows, SIGNAL(triggered()),
				this, SLOT(slot_StyleWindows()));
	connect( m_ui->actionWindows_XP, SIGNAL(triggered()),
				this, SLOT(slot_StyleWindowsXP()));
	connect( m_ui->actionWindows_Vista, SIGNAL(triggered()),
				this, SLOT(slot_StyleWindowsVista()));


	m_dialogSettings = new STDialogSettings(this);
	
	connect( m_ui->actionSettings, SIGNAL(triggered()),
			m_dialogSettings, SLOT(slot_show()));
			
	
	m_dialogAbout = new QDialog();
	m_dialogAbout->setModal(true);
	
	m_ui_about = new UIAbout();
	m_ui_about->setupUi(m_dialogAbout);
	
	
	QString s = m_ui_about->textEdit->toHtml();
	m_ui_about->textEdit->setText( s.replace("VERSION", QString("%1.%2.%3.%4").arg(VERSION_MAJOR).arg(VERSION_MINOR).arg(VERSION_REV).arg(VERSION_TAG) ));
	
	connect( m_ui->actionAbout, SIGNAL( triggered()),
			m_dialogAbout, SLOT(show()));
			
	connect( m_ui->actionOpen_Torrent_Files, SIGNAL( triggered()),
			this, SLOT(slot_getOpenFileNames()));

	connect( m_ui->action_Quit, SIGNAL( triggered()),
			qApp, SLOT(quit()));
	connect(m_trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
            this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));


	setup_timer();
	

}


STWidgetMainWindow::~STWidgetMainWindow()
{
} 

/*!
    \fn STWidgetMainWindow::slot_StyleWindows
 */
void STWidgetMainWindow::slot_StyleWindows()
{
	QApplication::setStyle( new QWindowsStyle());
	write_style_settings("QWindowsStyle");
}


/*!
    \fn STWidgetMainWindow::slot_StyleWindowsXP()
 */
void STWidgetMainWindow::slot_StyleWindowsXP()
{
	#ifdef Q_WS_WIN
		if(QSysInfo::WindowsVersion == QSysInfo::WV_XP) {
			QApplication::setStyle( new QWindowsXPStyle());
			write_style_settings("QWindowsXPStyle");
		}
	#endif
}


/*!
    \fn STWidgetMainWindow::slot_StyleWindowsVista()
 */
void STWidgetMainWindow::slot_StyleWindowsVista()
{
	#ifdef Q_WS_WIN
		if(QSysInfo::WindowsVersion == QSysInfo::WV_VISTA) {
		// TODO: waiting for QT 4.3
//			QApplication::setStyle( new QWindowsVistaStyle());
//			writeStyleSettings("QWindowsVistaStyle");
		}
	#endif
}


/*!
    \fn STWidgetMainWindow::slot_StyleCleanlooks()
 */
void STWidgetMainWindow::slot_StyleCleanlooks()
{
	QApplication::setStyle( new QCleanlooksStyle());
	write_style_settings("QCleanlooksStyle");
}


/*!
    \fn STWidgetMainWindow::slot_StylePlastique()
 */
void STWidgetMainWindow::slot_StylePlastique()
{
	QApplication::setStyle( new QPlastiqueStyle());
	write_style_settings("QPlastiqueStyle");
}


/*!
    \fn STWidgetMainWindow::slot_StyleMac()
 */
void STWidgetMainWindow::slot_StyleMac()
{
	#ifdef Q_WS_MAC
    	QApplication::setStyle( new QMacStyle());
    	write_style_settings("QMacStyle");
    #endif
}


/*!
    \fn STWidgetMainWindow::slot_StyleCDE()
 */
void STWidgetMainWindow::slot_StyleCDE()
{
    QApplication::setStyle( new QCDEStyle());
    write_style_settings("QCDEStyle");
}

/*!
    \fn STWidgetMainWindow::slot_StyleMotif()
 */
void STWidgetMainWindow::slot_StyleMotif()
{
	QApplication::setStyle( new QMotifStyle());
	write_style_settings("QMotifStyle");
}

void STWidgetMainWindow::write_settings()
{
	STSettings::set_value("MainWindow", "size", size());
	STSettings::set_value("MainWindow", "pos", pos());
	STSettings::set_value("MainWindow", "lastOpenDir", m_lastOpenDir);
	STSettings::set_value("MainWindow", "balloonOnHideEnabled", m_balloonOnHideEnabled);
}

void STWidgetMainWindow::read_settings()
{
     resize(STSettings::value("MainWindow", "size", QSize(640, 480)).toSize());
     move(STSettings::value("MainWindow", "pos", QPoint(10, 10)).toPoint());
     m_lastOpenDir = STSettings::value("MainWindow", "lastOpenDir", QDir::fromNativeSeparators (QDir::homePath())).toString();
     
     QString style = STSettings::value("MainWindow", "style", "QPlastiqueStyle").toString();
     
     m_balloonOnHideEnabled = STSettings::value("MainWindow", "balloonOnHideEnabled", true).toBool();
     
	if( style == "QPlastiqueStyle") 
		slot_StylePlastique();
	if( style == "QCleanlooksStyle") 
		slot_StyleCleanlooks();
	if( style == "QWindowsStyle") 
		slot_StyleWindows();
	if( style == "QWindowsXPStyle") 
		slot_StyleWindowsXP();
	if( style == "QWindowsVistaStyle") 
		slot_StyleWindowsVista();
	if( style == "QMacStyle") 
		slot_StyleMac();
	if( style == "QMotifStyle")
		slot_StyleMotif();
	if( style == "QCDEStyle")
		slot_StyleCDE();
 }

/*!
    \fn STWidgetMainWindow::setSTSession(STSession *sess)
 */
void STWidgetMainWindow::set_session(STSession *sess)
{
	m_session = sess;
	m_session->slot_setSelectedMITab(0);
	
	connect(m_session, SIGNAL( signal_alert_send(const QString &) ),
				m_ui->widget_log, SLOT( slot_append(const QString &)   ) );
	
	connect(m_session, SIGNAL( signal_debug_send(const QString &) ),
					m_ui->widget_debug, SLOT( slot_append(const QString &)   ) );
	
	connect( m_session, SIGNAL( signal_parsed()),
			this, SLOT( slot_timer_reset()));
	
	connect( m_ui->widget_downloads->get_listWidget(), SIGNAL(itemSelectionChanged ()),	
			m_session, SLOT( slot_doit()));

	connect( m_ui->tabWidget_down, SIGNAL(currentChanged(int)),
			m_session, SLOT(slot_setSelectedMITab(int)));			
	
	connect( m_ui->widget_downloads, SIGNAL( signal_selected(QString)),
			m_session, SLOT(slot_setSelectedTorrentHash(QString )));
			
	connect( m_ui->widget_downloads, SIGNAL( signal_selected(QString)),
			m_session, SLOT( slot_doit() ));
		
	connect( m_ui->widget_downloads->get_listWidget()->selectionModel(), SIGNAL( selectionChanged ( const QItemSelection & , const QItemSelection &  )),
			m_session, SLOT( slot_doit() ));
			
	connect( m_dialogSettings, SIGNAL(signal_changed( )),
			m_session, SLOT(slot_apply_settings( )));
			
	connect( m_ui->actionPause, SIGNAL(triggered()),
			m_session, SLOT( slot_pauseTorrent() ));
	connect( m_ui->actionResume, SIGNAL(triggered()),
			m_session, SLOT( slot_resumeTorrent() ));
	connect( m_ui->actionRemove, SIGNAL(triggered()),
			m_session, SLOT( slot_removeTorrent() ));
			
	connect( m_ui->widget_downloads->m_actionPause, SIGNAL(triggered()),
			m_session, SLOT( slot_pauseTorrent() ));
	connect( m_ui->widget_downloads->m_actionResume, SIGNAL(triggered()),
			m_session, SLOT( slot_resumeTorrent() ));
	connect( m_ui->widget_downloads->m_actionRemove, SIGNAL(triggered()),
			m_session, SLOT( slot_removeTorrent() ));
}


/*!
    \fn STWidgetMainWindow::writeStyleSettings(QString stylename)
 */
void STWidgetMainWindow::write_style_settings(QString stylename)
{
	STSettings::set_value("MainWindow", "style", stylename);
}

/*!
    \fn STWidgetMainWindow::slot_getOpenFileNames()
 */
void STWidgetMainWindow::slot_getOpenFileNames()
{
	QStringList files = QFileDialog::getOpenFileNames(
                         this,
                         "Select one or more files to open",
                         m_lastOpenDir,
                         "Torrent Files (*.torrent)", 0, QFileDialog::DontUseNativeDialog);

	if(files.count() > 0)
	m_lastOpenDir = files.at(0);
		
    m_session->slot_addTorrentFiles(files);
}



void STWidgetMainWindow::dropEvent(QDropEvent *event)
{
 const QMimeData *mimeData = event->mimeData();
 
 	QStringList l;
 	for(int i=0; i < mimeData->urls().count() ; i++) {

 		l.append(mimeData->urls().at(i).toLocalFile());
 	}
    event->acceptProposedAction();
    m_session->slot_addTorrentFiles(l, false, "");
}

void STWidgetMainWindow::dragEnterEvent(QDragEnterEvent *event)
{
    event->acceptProposedAction();
}

void STWidgetMainWindow::dragMoveEvent(QDragMoveEvent *event)
{
    event->acceptProposedAction();
}

void STWidgetMainWindow::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
    switch (reason) {
    case QSystemTrayIcon::Trigger:
       if(   isVisible()) {
        	hide();
        } else {
        	show();
        }
        break;
    case QSystemTrayIcon::DoubleClick:
        break;
    case QSystemTrayIcon::MiddleClick:
        //showMessage();
        break;
    default:
        ;
    }
}

/*!
    \fn STWidgetMainWindow::aboutToQuit()
 */
void STWidgetMainWindow::slot_about_to_quit()
{
	STff::debug(D_INF , "void STWidgetMainWindow::slot_about_to_quit");
	m_trayIcon->hide();
	m_session->about_to_quit();
	write_settings();
}



/*!
    \fn STWidgetMainWindow::closeEvent ( QCloseEvent * event )
 */
void STWidgetMainWindow::closeEvent ( QCloseEvent * event )
{
	if (m_trayIcon->isVisible()) {
        QString text  = tr("The program will keep running in the \n"
                                    "system tray. To terminate the program, \n"
                                    "choose File -> Quit\nTo disable this balloon, right click on trayicon.");

		if( m_balloonOnHideEnabled ) {
        	m_trayIcon->showMessage("", text, QSystemTrayIcon::NoIcon,
                          10 * 1000);
		}

                                 
        hide();
        event->ignore();
        return;
    } else {

	}

	slot_about_to_quit();
	event->accept();
}


/*!
    \fn STWidgetMainWindow::slot_balloonOnHideTrigger()
 */
void STWidgetMainWindow::slot_balloon_on_hide()
{
    m_balloonOnHideEnabled = !m_balloonOnHideEnabled;
    if( m_balloonOnHideEnabled) {
    	actionBHide->setText("disable tray balloon on hide");
    } else {
    	actionBHide->setText("enable tray balloon on hide");
    }
}



/*!
    \fn STWidgetMainWindow::setupTimer()
 */
void STWidgetMainWindow::setup_timer()
{
	m_timer_counter = 0;
	
	connect( &m_timer, SIGNAL(timeout()),
			this, SLOT(slot_on_timer()));
    m_timer.start(110);
    
}

/*!
    \fn STWidgetMainWindow::slot_on_timer()
 */
void STWidgetMainWindow::slot_on_timer()
{
	if( m_timer_counter == 0 ) 
	{
		m_timer_counter = 1234;
			
		if( isHidden() || isMinimized() ) {
			m_session->set_mw_hidden(true);
			m_timer_counter = 115;
			return;
		} else {
			m_session->set_mw_hidden(false);
		}
		
		if( !m_session->mutex_tryLock()) {
			STff::debug(D_INF, "in slot_on_timer, tryLock == false");
			m_timer_counter = 0;
			return;
		}
		
		STff::debug(D_INF, "im slot_on_timer");
		map_string_stringlist	map_torrents	= m_session->get_map_torrents();
		map_int_stringlist		map_files		= m_session->get_map_files();
		map_int_stringlist		map_peers		= m_session->get_map_peers();
		map_string_stringlist	map_status	= m_session->get_map_status();
		
		map_string_string map_session_stats = m_session->get_map_session_stats();
		
		update_status_bar(map_session_stats);
		
		m_ui->widget_files->slot_update_model( map_files );
		
		m_ui->widget_peers->slot_update_model( map_peers );
		
		m_ui->widget_downloads->slot_update_model( map_torrents );
		
		m_ui->widget_status->slot_update_model( map_status );
		
		m_session->mutex_unlock();
		
		m_timer_counter = 25;
	} else  if (m_timer_counter > 0) 
	{

    	m_timer_counter--;
    }
}


/*!
    \fn STWidgetMainWindow::slot_doit()
 */
void STWidgetMainWindow::slot_doit()
{
	slot_timer_reset();
	m_session->slot_doit();
}


/*!
    \fn STWidgetMainWindow::hideEvent ( QHideEvent * event )
 */
void STWidgetMainWindow::hideEvent ( QHideEvent * event )
{
	STff::debug(D_INF, "im hideEvent");
	event->accept();
}


/*!
    \fn STWidgetMainWindow::showEvent ( QShowEvent * event )
 */
void STWidgetMainWindow::showEvent ( QShowEvent * event )
{
    STff::debug(D_INF, "im showEvent");
    
    slot_doit();
    
	event->accept();
}


/*!
    \fn STWidgetMainWindow::slot_timer_reset()
 */
void STWidgetMainWindow::slot_timer_reset()
{
    m_timer_counter = 0;
}


/*!
    \fn STWidgetMainWindow::update_status_bar(map_string_string m)
 */
void STWidgetMainWindow::update_status_bar(map_string_string m)
{
	QString message;
	
	message += "down speed: " + m["download_rate"] + "/s | ";
	message += "up speed: " +  m["upload_rate"] + "/s | ";
	
	message += "downloaded: " + m["total_download"] + " | ";
	message += "uploaded: " +  m["total_upload"] + " | ";
	
	message += "connections: " +  m["num_connections"] + " | ";
	message += "uploads: " +  m["num_uploads"] + " | ";
	
	message += "peers: " +  m["num_peers"] + " | ";
	
	message += "dht nodes: " +  m["dht_nodes"] + " | ";
	
	message += "port: " +  m["listen_port"] + "";
	statusBar()->showMessage(message);
}


/*!
    \fn st::widgets::STWidgetMainWindow::check_updates()
 */
bool st::widgets::STWidgetMainWindow::check_updates()
{


}


