#ifndef STWIDGETMAINWINDOW_H
#define STWIDGETMAINWINDOW_H

#include <QMainWindow>

#include <QWindowsStyle>
#include <QWindowsXPStyle>

#include <QCDEStyle>
#include <QMotifStyle>

#ifdef Q_WS_MAC
#include <QMacStyle>
#endif

#include <QPlastiqueStyle>
#include <QCleanlooksStyle>
#include <QMessageBox>
#include <QFileDialog>

#include <QCloseEvent>
#include <QTimer>
#include <QSystemTrayIcon>

#include <QUrl>

#include <QTemporaryFile>

#include <QNetworkProxy>
#include <QHttp>

#include "ui/uiabout.h"
#include "ui/uimainwindow.h"
#include "stdialogsettings.h"
#include "stsession.h"
#include "stsettings.h"
using namespace Ui;

using namespace st::core;
using namespace st::widgets;
namespace st {
namespace widgets {
/**
	@author sfinks <sfinks2@gmail.com>
*/
class STWidgetMainWindow : public QMainWindow
{
	Q_OBJECT
public:
    STWidgetMainWindow(QWidget *parent = 0);

    ~STWidgetMainWindow();
    void set_session(STSession *sess);
    void dragMoveEvent(QDragMoveEvent *event);
    void dragEnterEvent(QDragEnterEvent *event);
    void showEvent ( QShowEvent * event );
	bool check_updates();
	    
protected:
	   void closeEvent ( QCloseEvent * event );
private:
	void write_settings();
	void read_settings();
    void write_style_settings(QString stylename);
    void dropEvent(QDropEvent *event);
    void setup_timer();
    void hideEvent ( QHideEvent * event );
    QSystemTrayIcon *m_trayIcon;
    QMenu			*m_trayMenu;
    QTimer		m_timer;
	unsigned int 	m_timer_counter;
	QDialog *m_dialog_update_notifier;

private:
	UIMainWindow *m_ui;
	STDialogSettings *m_dialogSettings;
	
	QDialog *m_dialogAbout;
	UIAbout *m_ui_about;
	STSession *m_session;
	QString m_lastOpenDir;
	
	QAction *actionHide, *actionShow;
	QAction *actionBHide;
	bool	m_balloonOnHideEnabled;
	
private slots:
    void slot_StyleMotif();
    void slot_StyleCDE();
    void slot_StyleMac();
    void slot_StylePlastique();
    void slot_StyleCleanlooks();
    void slot_StyleWindowsVista();
    void slot_StyleWindowsXP();
    void slot_StyleWindows();

    void slot_getOpenFileNames();
    
    void iconActivated(QSystemTrayIcon::ActivationReason reason);
    
    void slot_balloon_on_hide();
    
public slots:
    void slot_about_to_quit();
protected slots:
    void slot_on_timer();
    void slot_timer_reset();
private slots:
    void slot_doit();

private:
    void update_status_bar(map_string_string m);


};
}
}
#endif
